//ini-file processing for NONCOLAF.CPP
//(c)V.Glazkov, March 2016
#include <stdio.h>
#include <string.h>
#include"vector.h"

//global variables
extern vector3d dirH;
extern double a[4][4][4][4],I[4],gamma,Hstart,Hstop,Hstep,euler0[4];
extern int is_a[4][4][4][4],minsearch_flag,minsearch_grid;

//Functions declaration
int ini_read(void);//reads ini file
int ini_validate(void);//checks ini-file parameters for consistency

//ini-file processing parameters
#define NumberOfIniParams 12
#define _gamma 1
#define _I1 2
#define _I2 3
#define _I3 4
#define _Anisotropy 5
#define _Hdir 6
#define _Hstart 7
#define _Hstop 8
#define _Hstep 9
#define _minsearch 10
#define _minsearch_grid 11
#define _zero_approx 12

#define StrLength 200

#define _x 1
#define _y 2
#define _z 3

//Functions description
int _l_index(char ch)
	{
	if(ch=='1')
		return(1);
	if(ch=='2')
		return(2);
	if(ch=='3')
		return(3);
	return(0);
	}

int _l_projection(char ch)
	{
	if(ch=='x'||ch=='X')
		return _x;
	if(ch=='y'||ch=='Y')
		return _y;
	if(ch=='z'||ch=='Z')
		return _z;
	return(0);
	}

void spacekiller(char * string_in,char *string_out)
	{
	int i,j;
	j=0;
	for(i=0;(i<strlen(string_in)&&i<StrLength);i++)
		{
		if(string_in[i]=='\0'||string_in[i]=='\n')
			break;
		if(string_in[i]!=' ')
			{
			string_out[j]=string_in[i];
			j++;
			}
		}
	string_out[j]='\0';
	}

//Functions desscriptions
int ini_read(void)
{
FILE * inifile;
int counter, csum,i,j,k,l,conv;
char bufer[StrLength],bufer1[StrLength],anis[StrLength];
float ftmp,xtmp,ytmp,ztmp;

for(i=1;i<=3;i++)
	for(j=1;j<=3;j++)
		for(k=1;k<=3;k++)
			for(l=1;l<=3;l++)
				{
				a[i][j][k][l]=0.;
				is_a[i][j][k][l]=0;
				}

inifile=fopen("noncolaf.ini","r");

if(inifile==NULL)
	{
	printf("No INI file found\nCreating a template\n");
	inifile=fopen("noncolaf.ini","w");
	fprintf(inifile,"INI-file for NonColAF program\nPlease keep bracketed [...] names intact\nEnter your parameters after appropriate name\n\n[gamma:]\n\n[I1:]\n\n[I2:]\n\n[I3:]\n\n[AnisotropyStart:]\n\n[AnisotropyEnd:]\n\n[Hdir:] (Semicolon separated vector)\n\n[Hstart:]\n\n[Hstop:]\n\n[Hstep:]\n\n[minsearch flag:](1-global,0-local)\n\n[grid size:](for global minimum search)\n\n[starting approximation:](for local minimum search, semicolumn separated euler angles)\n");
	fclose(inifile);
	return(1);
	}

fseek(inifile, 0, SEEK_SET);

csum=counter=0;

while(feof(inifile)==0)
	{
	memset(bufer,0,sizeof(bufer));

	//read string from INI file
	fgets(bufer,sizeof(bufer),inifile);

	//analyse string for a variable name
	if(strstr(bufer,"[gamma:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_gamma;
		csum++;
		continue;
		}
	if(strstr(bufer,"[I1:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_I1;
		csum++;
		continue;
		}
	if(strstr(bufer,"[I2:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_I2;
		csum++;
		continue;
		}
	if(strstr(bufer,"[I3:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_I3;
		csum++;
		continue;
		}
	if(strstr(bufer,"[AnisotropyStart:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_Anisotropy;
		csum++;
		continue;
		}
	if(strstr(bufer,"[AnisotropyEnd:]")!=NULL)
		{
		if(counter==0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=0;
		continue;
		}
	if(strstr(bufer,"[Hdir:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_Hdir;
		csum++;
		continue;
		}
	if(strstr(bufer,"[Hstart:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_Hstart;
		csum++;
		continue;
		}
	if(strstr(bufer,"[Hstop:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_Hstop;
		csum++;
		continue;
		}
	if(strstr(bufer,"[Hstep:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_Hstep;
		csum++;
		continue;
		}
	if(strstr(bufer,"[minsearch flag:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_minsearch;
		csum++;
		continue;
		}
	if(strstr(bufer,"[grid size:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_minsearch_grid;
		csum++;
		continue;
		}
	if(strstr(bufer,"[starting approximation:]")!=NULL)
		{
		if(counter!=0) {printf("INI corrupted\n(Error before %s)\n",bufer);return(1);}
		counter=_zero_approx;
		csum++;
		continue;
		}

	//Now looking for a value for specified parameter
	//Simple numeric parameters

	if(counter!=0&&counter!=_Anisotropy&&counter!=_Hdir&&counter!=_zero_approx)
		{
		spacekiller(bufer,bufer1);

		if(strlen(bufer1)==0) continue;

		conv=sscanf(bufer1,"%e",&ftmp);
		if(conv==0||conv==EOF)
			{
			printf("INI corrupted\n");
			switch(counter)
				{
				case _gamma: printf("GAMMA ");
				case _I1: printf("I1 ");
				case _I2: printf("I2 ");
				case _I3: printf("I3 ");
				case _Hstart: printf("Hstart ");
				case _Hstop: printf("Hstop ");
				case _Hstep: printf("Hstep ");
				case  _minsearch: printf("Minsearch flag ");
				case  _minsearch_grid: printf("Minsearch grid size ");
				default: printf("conversion error\n");
				}
			return(1);
			}

		switch(counter)
			{
			case _gamma://gamma
				gamma=(double)ftmp;
				counter=0;
				break;
			case _I1://I1
				I[1]=(double)ftmp;
				counter=0;
				break;
			case _I2://I2
				I[2]=(double)ftmp;
				counter=0;
				break;
			case _I3://I3
				I[3]=(double)ftmp;
				counter=0;
				break;
			case _Hstart://Hstart
				Hstart=(double)ftmp;
				counter=0;
				break;
			case _Hstop://Hstop
				Hstop=(double)ftmp;
				counter=0;
				break;
			case _Hstep://Hstep
				Hstep=(double)ftmp;
				counter=0;
				break;
			case _minsearch://minsearch flag
				minsearch_flag=(int)ftmp;
				counter=0;
				break;
			case _minsearch_grid://global minsearch grid size
				minsearch_grid=(int)ftmp;
				counter=0;
				break;
			default:break;
			}
		}
	//Scanning Hdir vector and normalizing it
	if(counter==_Hdir)
		{
		spacekiller(bufer,bufer1);
		if(strlen(bufer1)==0) continue;

		conv=sscanf(bufer1,"%e;%e;%e",&xtmp,&ytmp,&ztmp);
		dirH=makevector((double)xtmp,(double)ytmp,(double)ztmp);

		if(conv==0||conv==EOF||mod(dirH)==0.)
			{
			printf("Invalid field direction specified\n");
			return(1);
			}
		dirH=(1./mod(dirH))*dirH;
		counter=0;
		}

	//Scanning zero approximation vector
	if(counter==_zero_approx)
		{
		spacekiller(bufer,bufer1);
		if(strlen(bufer1)==0) continue;

		conv=sscanf(bufer1,"%e;%e;%e",&xtmp,&ytmp,&ztmp);

		if(conv==0||conv==EOF)
			{
			printf("Invalid zero-approximation euler angles specified\n");
			return(1);
			}
		euler0[1]=(double)xtmp;
		euler0[2]=(double)ytmp;
		euler0[3]=(double)ztmp;

		counter=0;
		}

	//Reading anisotropy energy coefficients
	if(counter==_Anisotropy)
		{
		spacekiller(bufer,bufer1);
		if(strlen(bufer1)==0) continue;

		memset(anis,0,sizeof(anis));

		for(i=0;i<strlen(bufer1);i++)
			{
			if(bufer1[i]==';')
				break;
			anis[i]=bufer1[i];
			}
		if(i==strlen(bufer1))
			{
			printf("Invalid anisotropy energy: no numerical value specified or semicolon missing\n");
			return(1);
			}
		memset(bufer,0,sizeof(bufer));
		for(j=i+1;j<strlen(bufer1);j++)
			{
			bufer[j-i-1]=bufer1[j];
			if(bufer1[j]=='\0') break;
			}
		conv=sscanf(bufer,"%e",&ftmp);
		if(conv==0||conv==EOF)
			{
			printf("Invalid anisotropy energy: numerical value conversion error at %s term\n",anis);
			return(1);
			}
		if(strlen(anis)!=6)
			{
			printf("Invalid anisotropy energy: misprint in anisotropy term %s \n",anis);
			return(1);
			}
		//decifering a[i][j][k][l]=l_i_j l_k_l, i;k=1,2;3, j,l=x,y,z
		i=_l_index(anis[1]);
		j=_l_projection(anis[2]);
		k=_l_index(anis[4]);
		l=_l_projection(anis[5]);
		if(i*j*k*l==0)
			{
			printf("Invalid anisotropy energy: misprint in anisotropy term %s, should be in the form lialphaljbeta \n",anis);
			return(1);
			}
		if(is_a[i][j][k][l]||is_a[k][l][i][j])
			{
			printf("Invalid anisotropy energy: term %s (or equivalent) already specified \n",anis);
			return(1);
			}
			is_a[i][j][k][l]=1;
			a[i][j][k][l]=(double)ftmp;
		}
	}
	fclose(inifile);
	if(csum!=NumberOfIniParams)//csum shoul be equal to the number of scanned parameters
		{
		printf("INI corrupted\n Not all parameters specified\n");
		return(1);
		}

printf("INI succesfully processed\n");

return(0);
}

int ini_validate(void)
{
if(Hstep==0.)
	{
	printf("Invalid parameter: Hstep should be non-zero\n");
	return(2);
	}
if((Hstop-Hstart)*Hstep<0.)
	{
	printf("Invalid parameter: Hstop is unreachable (Hstart=%.2g;Hstop=%.2g;Hstep=%.2g)\n",(float)Hstart,(float)Hstop,(float)Hstep);
	return(2);
	}
if(minsearch_flag!=0&&minsearch_flag!=1)
	{
	printf("Invalid parameter: minsearch should be 1 (global) or 0 (local)\n");
	return(2);
	}
if(minsearch_grid<1&&minsearch_flag==1)
	{
	printf("Invalid parameter: minsearch grid should be above 1 at least\n");
	return(2);
	}
if(gamma*I[1]*I[2]*I[3]<=0.)
	{
	printf("Invalid parameter: gamma and I[i] constants should be positive\n");
	return(2);
	}

return(0);
}
#undef NumberOfIniParams
#undef _gamma
#undef _I1
#undef _I2
#undef _I3
#undef _Anisotropy
#undef _Hdir
#undef _Hstart
#undef _Hstop
#undef _Hstep
#undef _minsearch
#undef _minsearch_grid
#undef _zero_approx

#undef StrLength

#undef _x
#undef _y
#undef _z
