//Magnetic vectors and their derivatives
//for NONCOLAF.CPP
//(c)V.Glazkov, March 2016

#ifndef _MAGVECT
	#define _MAGVECT
	#include "vector.h"

//Functions declarations
	//i-th magnetic vector via Euler angles(classical definition)
	vector3d l_vec(int i, double theta, double phi, double psi, int * err);

	//first derivatives over Euler angles for field gradient calculations
	//i-th magnetic vector,j-th Euler angle
	vector3d dl_deuler(int i,int j, double theta, double phi, double psi, int * err);

	//first derivatives over rotation vector components
	//at equilibrium position
	//for dynamics equations
	//over j-th component
	vector3d dl_dphi(int j, vector3d l0, int * err);

	//second derivatives over rotation vector components
	//at equilibrium position
	//for dynamics equations
	//over j-th and k-th component
	vector3d d2l_dphi2(int j, int k, vector3d l0, int * err);


//Functions definitions
	vector3d l_vec(int i, double theta, double phi, double psi, int * err)
		{
		vector3d vec;
		int code=0;
		switch (i)
			{
			case 1:	//l1
				vec=makevector(cos(phi)*cos(psi)-cos(theta)*sin(phi)*sin(psi),sin(phi)*cos(psi)+cos(theta)*cos(phi)*sin(psi),sin(theta)*sin(psi));
				break;
			case 2:	//l2
				vec=makevector(-cos(phi)*sin(psi)-cos(theta)*sin(phi)*cos(psi),-sin(phi)*sin(psi)+cos(theta)*cos(phi)*cos(psi),sin(theta)*cos(psi));
				break;
			case 3: //l3
				vec=makevector(sin(theta)*sin(phi),-sin(theta)*cos(phi),cos(theta));
				break;
			default://wrong i!!!!
				code=1;
				vec=makevector(0.,0.,0.);
				break;
			}
		*err=code;
		return(vec);
		}

	vector3d dl_deuler(int i,int j, double theta, double phi, double psi, int * err)
		{
		vector3d vec;
		int code=0;

		if(i<1||i>3||j<1||j>3)//invalid parameters
			{
			code=1;
			vec=makevector(0.,0.,0.);

			*err=code;
			return(vec);
			}
		if(j==1)//dl[i]/dTheta
			{
			switch (i)
				{
				case 1:	//l1
					vec=makevector(sin(theta)*sin(phi)*sin(psi),-sin(theta)*cos(phi)*sin(psi),cos(theta)*sin(psi));
					break;
				case 2:	//l2
					vec=makevector(sin(theta)*sin(phi)*cos(psi),-sin(theta)*cos(phi)*cos(psi),cos(theta)*cos(psi));
					break;
				case 3: //l3
					vec=makevector(cos(theta)*sin(phi),-cos(theta)*cos(phi),-sin(theta));
					break;
				default://wrong i!!!!
					code=1;
					vec=makevector(0.,0.,0.);
					break;
				}
			}
		if(j==2)//dl[i]/dphi
			{
			switch (i)
				{
				case 1:	//l1
					vec=makevector(-sin(phi)*cos(psi)-cos(theta)*cos(phi)*sin(psi),cos(phi)*cos(psi)-cos(theta)*sin(phi)*sin(psi),0.);
					break;
				case 2:	//l2
					vec=makevector(sin(phi)*sin(psi)-cos(theta)*cos(phi)*cos(psi),-cos(phi)*sin(psi)-cos(theta)*sin(phi)*cos(psi),0.);
					break;
				case 3: //l3
					vec=makevector(sin(theta)*cos(phi),sin(theta)*sin(phi),0.);
					break;
				default://wrong i!!!!
					code=1;
					vec=makevector(0.,0.,0.);
					break;
				}

			}
		if(j==3)//dl[i]/dpsi
			{
			switch (i)
				{
				case 1:	//l1
					vec=makevector(-cos(phi)*sin(psi)-cos(theta)*sin(phi)*cos(psi),-sin(phi)*sin(psi)+cos(theta)*cos(phi)*cos(psi),sin(theta)*cos(psi));
					break;
				case 2:	//l2
					vec=makevector(-cos(phi)*cos(psi)+cos(theta)*sin(phi)*sin(psi),-sin(phi)*cos(psi)-cos(theta)*cos(phi)*sin(psi),-sin(theta)*sin(psi));
					break;
				case 3: //l3
					vec=makevector(0.,0.,0.);
					break;
				default://wrong!!!!
					code=1;
					vec=makevector(0.,0.,0.);
					break;
				}

			}
		*err=code;
		return (vec);
		}

	vector3d dl_dphi(int j, vector3d l0, int * err)
		{
		vector3d vec;
		int code=0;

		switch (j)
			{
			case 1://dl/dphi_x
				vec=makevector(0.,-l0.z,l0.y);
				break;
			case 2://dl/dphi_y
				vec=makevector(l0.z,0.,-l0.x);
				break;
			case 3://dl/dphi_z
				vec=makevector(-l0.y,l0.x,0.);
				break;
			default://wrong!!!
				code=1;
				vec=makevector(0.,0.,0.);
				break;
			}
		*err=code;
		return(vec);
		}

	double Kroeneker_delta (int i,int j)
		{
		if(i==j)
			return(1.);
		return(0.);
		}

	vector3d d2l_dphi2(int j, int k, vector3d l0, int * err)
		{
		double coeff,deriv[4];
		vector3d vec;
		int code=0;
		int i;

		if(j<1||j>3||k<1||k>3)
			{
			code=1;
			vec=makevector(0.,0.,0.);

			*err=code;
			return(vec);
			}
		for(i=1;i<=3;i++)
			deriv[i]=0.5*Kroeneker_delta(j,i)*component(l0,k)+0.5*Kroeneker_delta(k,i)*component(l0,j)-Kroeneker_delta(j,k)*component(l0,i);
		vec=makevector(deriv[1],deriv[2],deriv[3]);

		*err=code;
		return(vec);
		}

#endif